import re
import functools

from django.contrib.admindocs.views import simplify_regex
from django.core.urlresolvers import RegexURLPattern, RegexURLResolver


def get_urls(patterns, sort=False):
    urls = []
    found = get_patterns(patterns)

    for base, pattern in found:
        func = pattern.callback
        if isinstance(func, functools.partial):
            func = func.func

        func_name = get_name(func)
        regex = base + pattern.regex.pattern

        urls.append(dict(
            view="{0}.{1}".format(func.__module__, func_name),
            name=pattern.name or "",
            path=simplify_regex(regex),
            args=pattern.default_args,
        ))

    if sort:
        urls = sorted(urls, key=lambda x: x["path"])

    return urls


def get_name(func):
    if hasattr(func, "__name__"):
        return func.__name__
    elif hasattr(func, "__class__"):
        return "%s()" % func.__class__.__name__
    else:
        return re.sub(r" at 0x[0-9a-f]+", "", repr(func))


def get_patterns(patterns, base=""):
    for p in patterns:
        if isinstance(p, RegexURLPattern):
            yield (base, p)
        elif isinstance(p, RegexURLResolver):
            for p in get_patterns(p.url_patterns, base + p.regex.pattern):
                yield p
        else:
            raise TypeError(
                "%s does not appear to be a url pattern" % p,
            )



from django.conf.urls import include, url
from django.test import TestCase

from .utils import get_urls


def view(request, **kwargs):
    pass


extra = [
    url("^world/$", view),
]

nested = [
    url(r"^(?P<slug>\w+)/", include([
        url(r"^$", view, dict(x=1)),
        url(r"^history/$", view, dict(y=2)),
        url(r"^edit/$", view, dict(z=3)),
    ])),
]

urlpatterns = [
    url("^$", view),
    url("^product/(?P<slug>[^/]+)/$", view, name="product-detail"),
    url("^about/$", view, name="about"),
    url("^hello/", include(extra)),
    url("^page/", include(nested)),
]


class GetUrlsTest(TestCase):

    def test_get(self):
        urls = get_urls(urlpatterns)

        self.assertEqual(urls[0]["path"], "/")
        self.assertEqual(urls[0]["name"], "")
        self.assertEqual(urls[0]["view"], "urlmapper.tests.view")
        self.assertEqual(urls[0]["args"], {})

        self.assertEqual(urls[1]["path"], "/product/<slug>/")
        self.assertEqual(urls[1]["name"], "product-detail")

        self.assertEqual(urls[2]["path"], "/about/")
        self.assertEqual(urls[2]["name"], "about")

        self.assertEqual(urls[3]["path"], "/hello/world/")

        self.assertEqual(urls[4]["path"], "/page/<slug>/")
        self.assertEqual(urls[4]["args"], {"x": 1})

        self.assertEqual(urls[5]["path"], "/page/<slug>/history/")
        self.assertEqual(urls[5]["args"], {"y": 2})

        self.assertEqual(urls[6]["path"], "/page/<slug>/edit/")
        self.assertEqual(urls[6]["args"], {"z": 3})

    def test_sort(self):
        urls = get_urls(urlpatterns, sort=True)

        self.assertEqual(urls[0]["path"], "/")
        self.assertEqual(urls[1]["path"], "/about/")
        self.assertEqual(urls[2]["path"], "/hello/world/")



import codecs
from setuptools import setup, find_packages
from os.path import dirname, join, realpath


ROOT = realpath(dirname(__file__))


def read(path):
    with codecs.open(join(ROOT, path), encoding="utf-8") as f:
        return f.read()


version = __import__("urlmapper").__version__


setup(
    name="django-urlmapper",
    version=version,
    url="https://github.com/prestontimmons/django-urlmapper",
    description="Print registered urls in a readable format.",
    long_description=read("README.rst"),
    author="Preston Timmons",
    author_email="prestontimmons@gmail.com",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
    ],
    classifiers=[
        "Framework :: Django",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
    ],
)
